/** **************************************************************************
 * notification.c
 *
 * Copyright 2008 Bryan Ischo <bryan@ischo.com>
 * (Actually this is added by nick huang <nickhuang99@hotmail.com> by following
 * coding style of Mr. Bryan Ishcho for new AWS S3 api.)
 *
 * This file is part of libs3.
 *
 * libs3 is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, version 3 of the License.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of this library and its programs with the
 * OpenSSL library, and distribute linked combinations including the two.
 *
 * libs3 is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with libs3, in a file named COPYING.  If not, see
 * <http://www.gnu.org/licenses/>.
 *
 ************************************************************************** **/


#include <stdlib.h>
#include <string.h>
#include "libs3.h"
#include "request.h"

// Use a rather arbitrary max size for the document of 64K
#define BUCKET_NOTIFICATION_XML_DOC_MAXSIZE (64 * 1024)

/////////////////////////////////////////
// added by nick
typedef struct SetBucketNotificationData
{
    S3ResponsePropertiesCallback *responsePropertiesCallback;
    S3ResponseCompleteCallback *responseCompleteCallback;
    void *callbackData;

    int notificationXmlDocumentLen;
    char notificationXmlDocument[BUCKET_NOTIFICATION_XML_DOC_MAXSIZE];
    int notificationXmlDocumentBytesWritten;

} SetBucketNotificationData;


///////////////////////////
// added by nick
static void setBucketNotificationCompleteCallback(S3Status requestStatus,
                                   const S3ErrorDetails *s3ErrorDetails,
                                   void *callbackData)
{
    SetBucketNotificationData *paData = (SetBucketNotificationData *) callbackData;

    (*(paData->responseCompleteCallback))
        (requestStatus, s3ErrorDetails, paData->callbackData);
    free(paData);
}

///////////////////////////////////
// added by nick
static int setBucketNotificationDataCallback(int bufferSize, char *buffer, void *callbackData)
{
    SetBucketNotificationData *paData = (SetBucketNotificationData *) callbackData;

    int remaining = (paData->notificationXmlDocumentLen -
                     paData->notificationXmlDocumentBytesWritten);

    int toCopy = bufferSize > remaining ? remaining : bufferSize;

    if (!toCopy)
    {
        return 0;
    }

    memcpy(buffer, &(paData->notificationXmlDocument
                     [paData->notificationXmlDocumentBytesWritten]), toCopy);

    paData->notificationXmlDocumentBytesWritten += toCopy;

    return toCopy;
}
////////////////////////////
// added by nick
static S3Status setBucketNotificationPropertiesCallback
    (const S3ResponseProperties *responseProperties, void *callbackData)
{
    SetBucketNotificationData *paData = (SetBucketNotificationData *) callbackData;

    return (*(paData->responsePropertiesCallback))
        (responseProperties, paData->callbackData);
}

////////////////////////////////////////////////
// added by nick
void S3_get_bucket_notification(const S3BucketContext *bucketContext,
                   S3RequestContext *requestContext,
                   const S3GetBucketNotificationHandler *handler, void *callbackData)
{
    // Set up the RequestParams
    RequestParams params =
    {
        HttpRequestTypeGET,                           // httpRequestType
        { bucketContext->bucketName,                  // bucketName
          bucketContext->protocol,                    // protocol
          bucketContext->uriStyle,                    // uriStyle
          bucketContext->accessKeyId,                 // accessKeyId
          bucketContext->secretAccessKey },           // secretAccessKey
        0,                                            // key
        0,                                            // queryParams
        "notification",                                     // subResource
        0,                                            // copySourceBucketName
        0,                                            // copySourceKey
        0,                                // getConditions
        0,                                    // startByte
        0,                                    // byteCount
        0,                                            // putProperties
        handler->responseHandler.propertiesCallback,  // propertiesCallback
        0,                                            // toS3Callback
        0,                                            // toS3CallbackTotalSize
        handler->getBucketNotificationDataCallback,         // fromS3Callback
        handler->responseHandler.completeCallback,    // completeCallback
        callbackData                                  // callbackData
    };

    // Perform the request
    request_perform(&params, requestContext);
}


////////////////////////////////////////////
// added by nic
void S3_set_bucket_notification(const S3BucketContext *bucketContext, const char* notificationBuf, int policyBufSize,
                S3RequestContext *requestContext,
                const S3ResponseHandler *handler, void *callbackData)
{

    SetBucketNotificationData *data = (SetBucketNotificationData *) malloc(sizeof(SetBucketNotificationData));
    if (!data) {
        (*(handler->completeCallback))(S3StatusOutOfMemory, 0, callbackData);
        return;
    }

    data->responsePropertiesCallback = handler->propertiesCallback;
    data->responseCompleteCallback = handler->completeCallback;
    data->callbackData = callbackData;

    data->notificationXmlDocumentBytesWritten = 0;
    strncpy(data->notificationXmlDocument, notificationBuf, BUCKET_NOTIFICATION_XML_DOC_MAXSIZE);
    data->notificationXmlDocumentLen = notificationBuf;

    // Set up the RequestParams
    RequestParams params =
    {
        HttpRequestTypePUT,                           // httpRequestType
        { bucketContext->bucketName,                  // bucketName
          bucketContext->protocol,                    // protocol
          bucketContext->uriStyle,                    // uriStyle
          bucketContext->accessKeyId,                 // accessKeyId
          bucketContext->secretAccessKey },           // secretAccessKey
        0,                                            // key
        0,                                            // queryParams
        "notification",                                     // subResource
        0,                                            // copySourceBucketName
        0,                                            // copySourceKey
        0,                                            // getConditions
        0,                                            // startByte
        0,                                            // byteCount
        0,                                            // putProperties
        &setBucketNotificationPropertiesCallback,           // propertiesCallback
        &setBucketNotificationDataCallback,                 // toS3Callback
        data->notificationXmlDocumentLen,                   // toS3CallbackTotalSize
        0,                                            // fromS3Callback
        &setBucketNotificationCompleteCallback,             // completeCallback
        data                                          // callbackData
    };

    // Perform the request
    request_perform(&params, requestContext);
}
